//+------------------------------------------------------------------+
//|                                                      jbtest3.mq4 |
//|                                    Copyright  2009, Johan Brits |
//|                                       http://www.besharewise.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, Johan Brits"
#property link      "http://www.besharewise.com"

#define MAGICMA  20090601

extern double Lots               = 0.01;
extern double MaximumRisk        = 0.02;

extern int TakeProfit = 800;
extern int StopLoss = 200;
extern int TrailingStop=400;
extern int TS.Min.Move.Pips = 1;

extern int Slippage = 5;

extern int StartHour = 0;  // No significant gain here at present
extern int EndHour   = 24; // No significant gain here at present

extern double DecreaseFactor = 3;
extern double ShortEma       = 21; // Best on Alpari Live data - originally 23
extern double LongEma        = 40; // Best on Alpari Live data - originally 38

extern int MAMode = 1; // 0=SMA 1=EMA 2=Smoothed 3=Linear Weighted - EMA rules (usually) on Cable

string strComment = "jbtest3 1.01 EURJPY H1";
int iPeriod;
string strSymbol;

int Real.TakeProfit, Real.StopLoss, Real.Slippage, Real.TrailingStop, Real.TS.Min.Move.Pips;

 

//+------------------------------------------------------------------+
//| Supporting functions                                             |
//+------------------------------------------------------------------+

void MoveTSPerMNwithFullPip(int iMN, int iTS)
{
// Changed to not default a SL

int icnt, itotal;
double dTS, dMinMove, dActualMove;

dMinMove = (Point*iTS)+(Real.TS.Min.Move.Pips*Point);
dActualMove = (Point*iTS);

itotal=OrdersTotal();

   for(icnt=0;icnt<itotal;icnt++) 
     {                               // order loop boundary
      OrderSelect(icnt, SELECT_BY_POS, MODE_TRADES);
      
       // check for opened position, symbol & MagicNumber
      if(OrderType()==OP_SELL && OrderSymbol()==strSymbol  && OrderMagicNumber()==iMN)  
        {     
         if((OrderOpenPrice() - Ask) > (dActualMove))
          {         
           if((OrderStopLoss() - Ask) > dMinMove)  
              {
                dTS = NormalizeDouble(Ask+dActualMove, Digits);
                OrderModify(OrderTicket(),OrderOpenPrice(), dTS, OrderTakeProfit(),0,Yellow);
              }
          }
      }
        
        
       if(OrderType()==OP_BUY && OrderSymbol()==strSymbol  && OrderMagicNumber()==iMN)  
        {   
         if((Bid - OrderOpenPrice()) > dActualMove)
          {     
            if((Bid - OrderStopLoss()) > dMinMove)
              {
                dTS = NormalizeDouble(Bid-dActualMove, Digits);
                OrderModify(OrderTicket(),OrderOpenPrice(), dTS,OrderTakeProfit(),0,Green);
              }
          }
              
        }

     }  // order loop boundary

return(0);
     
}


//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==strSymbol && OrderMagicNumber()==MAGICMA)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
 
//---- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=strSymbol || OrderType()>OP_SELL || OrderMagicNumber() != MAGICMA) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1)  lot=NormalizeDouble(lot*losses,2);//lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//---- return lot size
   lot = MathMax(lot, MarketInfo(strSymbol, MODE_MINLOT));
   
   return(lot);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double Ema;
   int    res;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   Ema=iMA(NULL,0,ShortEma,LongEma,MAMode,PRICE_CLOSE,0);
   double SEMA1=iMA(NULL,0,ShortEma,0,MAMode,PRICE_CLOSE,1);
   double LEMA1=iMA(NULL,0,LongEma,0,MAMode,PRICE_CLOSE,1);
   double SEMA2=iMA(NULL,0,ShortEma,0,MAMode,PRICE_CLOSE,2);
   double LEMA2=iMA(NULL,0,LongEma,0,MAMode,PRICE_CLOSE,2);

//---- sell conditions
  
   if (SEMA1<LEMA1 && SEMA2>LEMA2) 
     {
      res=OrderSend(strSymbol,OP_SELL,LotsOptimized(),Bid,Real.Slippage,Bid+Real.StopLoss*Point,Bid-Real.TakeProfit*Point,strComment,MAGICMA,0,Red);
      return;
     }
//---- buy conditions
  
    if (SEMA1>LEMA1 && SEMA2<LEMA2) 
     {
      res=OrderSend(strSymbol,OP_BUY,LotsOptimized(),Ask,Real.Slippage,Ask-Real.StopLoss*Point,Ask+Real.TakeProfit*Point,strComment,MAGICMA,0,Blue);
      return;
     }
//----
  }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
   double Ema;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   Ema=iMA(NULL,0,ShortEma,LongEma,MAMode,PRICE_CLOSE,0);
   double SEMA1=iMA(NULL,0,ShortEma,0,MAMode,PRICE_CLOSE,1);
   double LEMA1=iMA(NULL,0,LongEma,0,MAMode,PRICE_CLOSE,1);
   double SEMA2=iMA(NULL,0,ShortEma,0,MAMode,PRICE_CLOSE,2);
   double LEMA2=iMA(NULL,0,LongEma,0,MAMode,PRICE_CLOSE,2);
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if(OrderMagicNumber()!=MAGICMA || OrderSymbol()!=strSymbol) continue;
      //---- check order type 
      if(OrderType()==OP_BUY)
        {
        // if(Open[1]>Ema && Close[1]<Ema)
         if (SEMA1<LEMA1 && SEMA2>LEMA2)  OrderClose(OrderTicket(),OrderLots(),Bid,3,White);
         break;
        }
      if(OrderType()==OP_SELL)
        {
        // if(Open[1]<Ema && Close[1]>Ema)
        if (SEMA1>LEMA1 && SEMA2<LEMA2)  OrderClose(OrderTicket(),OrderLots(),Ask,3,White);
         break;
        }
     }
//----
  }
//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
//---- calculate open orders by current symbol
    
  
   iPeriod = Period();
   strSymbol = Symbol();
   
   
    if(StringFind(strSymbol, "EURJPY", 0)<0)
     {
      Print("Symbol set to: ", Symbol(), " is not correct for this EA!");
      Alert("Symbol set to: ", Symbol(), " is not correct for this EA!");
      return(3);  
     }
     
    if(iPeriod != PERIOD_H1) // Period of the chart should be same as signal period
     {
      Print("Period set to: ", iPeriod, " is not correct for this EA!");
      Alert("Period set to: ", iPeriod, " is not correct for this EA!");
      return(1);  
     }
     
     
  // Handle full/sub-pip 
   if (Digits == 4) 
     {
       Real.Slippage = Slippage;
       Real.TakeProfit = TakeProfit;
       Real.StopLoss = StopLoss;
       Real.TrailingStop = TrailingStop;
       Real.TS.Min.Move.Pips = TS.Min.Move.Pips;
     }
  
   if (Digits == 5) 
     {
       Real.Slippage = Slippage*10;
       Real.TakeProfit = TakeProfit*10;
       Real.StopLoss = StopLoss*10;
       Real.TrailingStop = TrailingStop*10;
       Real.TS.Min.Move.Pips = TS.Min.Move.Pips*10;
     }
     
   // Call Trailing Stop check every tick
   if (TrailingStop!=0) MoveTSPerMNwithFullPip(MAGICMA, Real.TrailingStop); 
     
     
   CheckForClose();
   
   
   
   if (Hour() < StartHour || Hour() > EndHour) return (0);
   
   CheckForOpen();

//----
  }
//+------------------------------------------------------------------+